#import "BitmapTexturePlugin.h"
#import "Render.h"

@interface BitmapTexturePlugin ()

@property (nonatomic) NSObject<FlutterTextureRegistry> *textures;
@property (nonatomic) NSMutableDictionary<NSNumber *, Render *> *renders;
@property (nonatomic) NSLock *glock;  // The global lock to lock all sub texture threads.

@end

@implementation BitmapTexturePlugin

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar> *)registrar {
  FlutterMethodChannel *channel = [FlutterMethodChannel
      methodChannelWithName:@"bitmap"
      binaryMessenger:[registrar messenger]];
  BitmapTexturePlugin *instance = [[BitmapTexturePlugin alloc] initWithTexture:[registrar textures]];
  [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result {
  if ([@"i" isEqualToString:call.method]) {
    int width = [call.arguments[@"width"] intValue];
    int height = [call.arguments[@"height"] intValue];

    NSInteger __block id = 0;  // Register TextureId
    Render *render = [[Render alloc] initWithCallback:^() {
      [self.textures textureFrameAvailable:id];
    } width:width height:height glock:_glock];
    id = (NSInteger) [_textures registerTexture:render];
    [render setId:id];
    _renders[@(id)] = render;
    result(@(id));
  }
  else if ([@"r" isEqualToString:call.method]) {  // render
    NSNumber *textureId = call.arguments[@"textureId"];
    NSString *path = call.arguments[@"path"];
    int width = [call.arguments[@"width"] intValue];
    int height = [call.arguments[@"height"] intValue];
    int srcWidth = [call.arguments[@"srcWidth"] intValue];
    int srcHeight = [call.arguments[@"srcHeight"] intValue];
    int fit = [call.arguments[@"fit"] intValue];
    NSString *bitmap = call.arguments[@"bitmap"];
    bool findCache = [call.arguments[@"findCache"] boolValue];
        
    if ([textureId intValue] == -1) {
      NSInteger __block id = 0;  // Register TextureId
      Render *render = [[Render alloc] initWithCallback:^() {
        [self.textures textureFrameAvailable:id];
      } width:width height:height glock:_glock];
      id = (NSInteger) [_textures registerTexture:render];
      [render setId:id];
      [render r:result path:path width:width height:height srcWidth:srcWidth srcHeight:srcHeight fit:fit bitmap:bitmap findCache:findCache];
      _renders[@(id)] = render;
    }
    else {
      Render *render = _renders[textureId];
      [render r:result path:path width:width height:height srcWidth:srcWidth srcHeight:srcHeight fit:fit bitmap:bitmap findCache:findCache];
    }
  }
  else if ([@"dl" isEqualToString:call.method]) {  // dispose list
    NSArray<NSNumber *> *textureIds = call.arguments[@"textureIds"];
    for (NSNumber *textureId in textureIds) {
      Render *render = _renders[textureId];
      [render d];
      [_renders removeObjectForKey:textureId];
      [_textures unregisterTexture:[textureId longValue]];
    }
    for (NSNumber *textureId in [_renders allKeys]) {
      NSLog(@"Key kept ... %d\n", [textureId intValue]);
    }
    result(nil);
  }
}

- (instancetype)initWithTexture:(NSObject<FlutterTextureRegistry> *)textures {
  self = [super init];
  if (self) {
    _textures = textures;
    _renders = [[NSMutableDictionary alloc] init];
    _glock = [[NSLock alloc] init];
  }
  return self;
}

@end
